
from Worzis_tools.проверка_элементов import содержит

РАЗДЕЛИТЕЛЬ_ПАР = "___,,,___"
РАЗДЕЛИТЕЛЬ_КЛЮЧЕЙ_ОТ_ЗНАЧЕНИЙ = "___:::___"

def записать_в_файл(путь: str, текст: str) -> str:
    with open(путь, "w", encoding='utf-8') as file:
        file.write(текст)

def считать_из_файла(путь: str) -> None:
    with open(путь, "r", encoding='utf-8') as file:
        return file.read()

def преобразовать_словарь_в_текст(словарь: dict):
    # Если разделитель встречается в одном из значений словаря 
    # то выдаст ошибку "В одном из значений словаря содержится разделитель"
    if содержит([РАЗДЕЛИТЕЛЬ_КЛЮЧЕЙ_ОТ_ЗНАЧЕНИЙ, РАЗДЕЛИТЕЛЬ_ПАР], list(словарь.values())):
        raise ValueError("В одном из значений словаря найден разделитель")
    результат = ""
    for ключ, значение in словарь.items():
        пара_в_тексте = f"{ключ}{РАЗДЕЛИТЕЛЬ_КЛЮЧЕЙ_ОТ_ЗНАЧЕНИЙ}{значение}"
        результат += пара_в_тексте + РАЗДЕЛИТЕЛЬ_ПАР
    результат = результат[:-len(РАЗДЕЛИТЕЛЬ_ПАР)]
    return результат

def записать_словари_в_файл(путь: str, список_словарей: list) -> None:
    общий_результат = ""
    for словарь in список_словарей:
        результат = преобразовать_словарь_в_текст(словарь)
        общий_результат += результат + "\n"
        # "\n" - специальный символ означающий завершение абзаца. Всё что написано после него - будет написано с новой строки
    записать_в_файл(путь, общий_результат)

def считать_словари_из_файла(путь: str) -> list:
    текст = считать_из_файла(путь)
    список_словарей = []
    for строка in текст.splitlines():
        пары = строка.split(РАЗДЕЛИТЕЛЬ_ПАР)
        словарь = {}
        for пара in пары:
            ключ_и_значение = пара.split(РАЗДЕЛИТЕЛЬ_КЛЮЧЕЙ_ОТ_ЗНАЧЕНИЙ)
            if len.ключ_и_значение != 2:
                return []
            ключ, значение = ключ_и_значение
            словарь[ключ] = значение
        список_словарей.append(словарь)

    return список_словарей

if __name__ == "__main__":
    # Windws разбивает путь на разные папки с помощью дроби \
    # Питон воспринимает этот символ как "специальный" и ожидает после него дополнительный специальный символ
    # Поэтому если просто скопировать путь фйла виндоуса в файл питона, питон вылетит с ошибкой
    # чтобы этого не допустить надо:
    #   1) либо проставить двойные дроби \\ вместо одинарных \
    #   2) либо проставить перед ковычками букву "r", но тогда не получиться использовать "f" строку

    путь = "C:\\Users\\user\\Desktop\\python\\one\\подключаемый_модуль\\Worzis_tools\\файл из питона.txt"
#    текст = "этот текст"
#
#    записать_в_файл(путь, "этот текст")
#    текст_из_файла = считать_из_файла(путь, текст)
#    print(текст_из_файла == текст)




    список_словарей = [{
        "имя": "Никита",
        "фамилия": "Тарасов",
        "должность": "Преподаватель",
        "возраст": "35"
    }, {
        "имя": "Даниил",
        "фамилия": "Махов",
        "должность": "Уборщик",
        "возраст": "19"
    }]
    
    записать_словари_в_файл(путь, список_словарей)
    результат = считать_словари_из_файла(путь)
    print(результат == список_словарей)




пройдено = 0

def содержит(что, где, игнор_регистра=False, стоп=False):
    # функция проверяет что хотя бы один элемент из "что" (или вся перменная целеком) содержится хоты бы в одном элементе из "где" (или является частью самой переменной "где")

    # функция возвращает True или False
    # переменные "что" и "где" могут быть двух видов:
        # 1. итерируемые объекты (кортеж или список), всеми элементами которых являются строки (текст)
        # 2. сама переменная может быть строкой (текстом)
        # при нарушении любого из этих пунктов будет вызвана ошибка с описанием этой проблемы
    # переменная "Игнорировать_регистр" может принимать только True или быть по умолчанию False. При передаче другого будет вызвана ошибка.
    # при получении в "что" или "где" пустых строк или пустых списков будет возвращаться False.
    if type(что) not in [list, str, tuple]:
        raise TypeError(f"Переменная 'что' может принять только список, кортеж или текст. Был получен {type(что)}")
    
    if type(игнор_регистра) != bool:
        raise TypeError(f"Переменная 'регистр' может принять True или False. Был получен {type(игнор_регистра)}")
    
    if type(где) not in [list, str, tuple]:
        raise TypeError(f"Переменная 'где' может принять только список, кортеж или текст. Был получен {type(где)}")
    
    if что in [[], (), ""]:
        if что == []:
            текст_ошибки = "список"
        elif что == ():
            текст_ошибки = "кортеж"
        elif что == "":
            текст_ошибки = "текст"
        raise TypeError(f"Переменная 'что' не может принять {текст_ошибки}")
    
    if где in [[], (), ""]:
        if где == []:
            текст_ошибки = "список"
        elif где == ():
            текст_ошибки = "кортеж"
        elif где == "":
            текст_ошибки = "текст"
        raise TypeError(f"Переменная 'где' не может принять {текст_ошибки}")
    
    for элемент in что:
        if type(элемент) != str:
            raise TypeError(f"Внутри переменной 'что' оказался не текстовый элемент - {элемент}")
        
    for элемент in где:
        if type(элемент) != str:
            raise TypeError(f"Внутри переменной 'где' оказался не текстовый элемент - {элемент}")
        
    if стоп:
        print("Стоп")

    if игнор_регистра:
        if type(что) == str:
            что = что.lower()
        else:
            # что = что.copy()
            # for индекс in range(len(что)):
            #     что[индекс] = что[индекс].lower()

            маленькие_буквы = []
            for элемент in что:
                маленькие_буквы.append(элемент.lower())
            что = маленькие_буквы

        if type(где) == str:
            где = где.lower()
        else:
            # где = где.copy()
            # for индекс in range(len(где)):
            #     где[индекс] = где[индекс].lower()

            маленькие_буквы = []
            for элемент in где:
                маленькие_буквы.append(элемент.lower())
            где = маленькие_буквы

    # текст - текст
    if type(что) == type(где) == str:
        return что in где
    
    # текст - множество
    if type(что) == str:
        for элемент in где:
            if что in элемент:
                return True
            
    # множество - текст
    if type(где) == str:
        for элемент in что:
            if элемент in где:
                return True
    
    # множество - множество
    if type(что) != str and type(где) != str:
        for элемент_что in что:
            for элемент_где in где:
                if элемент_что in элемент_где:
                    return True
    return False



def тесты():
    def тест(что ,где ,регистр, ожидаемый_ответ, название_теста):
        global пройдено

        try:
            результат = содержит(что , где , регистр)
        except:
            результат = "ошибка"
        if результат == ожидаемый_ответ:
            print(f"Тест '{название_теста}' пройден")
            пройдено += 1
        else:
            print()
            print(f"Тест '{название_теста}' не пройден")
            print(f"Ожидался ответ {ожидаемый_ответ}")
            print(f"Полученный ответ {результат}")
    
    
    название = "что список, где корректное, регистр корректный - не ожидаем ошибку"
    что = ["1", "2", "3"]
    где = ["1", "2", "3"]
    регистр = True
    ожидаемый_ответ = True
    тест(что ,где , регистр, ожидаемый_ответ, название)

    название = "что кортеж, где корректное, регистр корректный - не ожидаем ошибку"
    что = ("1", "2", "3")
    где = ["1", "2", "3"]
    регистр = True
    ожидаемый_ответ = True
    тест(что ,где , регистр, ожидаемый_ответ, название)

    название = "что текст, где корректное, регистр корректный - не ожидаем ошибку"
    что = "1"
    где = ["1", "2", "3"]
    регистр = True
    ожидаемый_ответ = True
    тест(что ,где , регистр, ожидаемый_ответ, название)


    название = "что другой тип, где корректное, регистр корректный - ожидаем ошибку"
    что = 1
    где = ["1", "2", "3"]
    регистр = True
    ожидаемый_ответ = "ошибка"
    тест(что ,где , регистр, ожидаемый_ответ, название)

    название = "что пустой список, где корректное, регистр корректный - ожидаем ошибку"
    что = []
    где = ["1", "2", "3"]
    регистр = True
    ожидаемый_ответ = "ошибка"
    тест(что ,где , регистр, ожидаемый_ответ, название)

    название = "что пустой кортеж, где корректное, регистр корректный - ожидаем ошибку"
    что = ()
    где = ["1", "2", "3"]
    регистр = True
    ожидаемый_ответ = "ошибка"
    тест(что ,где , регистр, ожидаемый_ответ, название)

    название = "что пустой текст, где корректное, регистр корректный - ожидаем ошибку"
    что = ""
    где = ["1", "2", "3"]
    регистр = True
    ожидаемый_ответ = "ошибка"
    тест(что ,где , регистр, ожидаемый_ответ, название)

    название = "в что один из элементов не строка, где корректный, регистр корректный - ожидаем ошибку"
    что = ["1", "2", 3]
    где = ["1", "2", "3"]
    регистр = True
    ожидаемый_ответ = "ошибка"
    тест(что ,где , регистр, ожидаемый_ответ, название)


    название = "что корректное, где список, регистр корректный - не ожидаем ошибку"
    что = ["1", "2", "3"]
    где = ["1", "2", "3"]
    регистр = True
    ожидаемый_ответ = True
    тест(что ,где , регистр, ожидаемый_ответ, название)

    название = "что корректное, где кортеж, регистр корректный - не ожидаем ошибку"
    что = ["1", "2", "3"]
    где = ("1", "2", "3")
    регистр = True
    ожидаемый_ответ = True
    тест(что ,где , регистр, ожидаемый_ответ, название)

    название = "что корректное, где текст, регистр корректный - не ожидаем ошибку"
    что = ["1", "2", "3"]
    где = "1"
    регистр = True
    ожидаемый_ответ = True
    тест(что ,где , регистр, ожидаемый_ответ, название)


    название = "что корректное, где другой тип, регистр корректный - ожидаем ошибку"
    что = ["1", "2", "3"]
    где = 3
    регистр = True
    ожидаемый_ответ = "ошибка"
    тест(что ,где , регистр, ожидаемый_ответ, название)

    название = "что корректное, где пустой список, регистр корректный - ожидаем ошибку"
    что = ["1", "2", "3"]
    где = []
    регистр = True
    ожидаемый_ответ = "ошибка"
    тест(что ,где , регистр, ожидаемый_ответ, название)

    название = "что корректное, где пустой кортеж, регистр корректный - ожидаем ошибку"
    что = ["1", "2", "3"]
    где = ()
    регистр = True
    ожидаемый_ответ = "ошибка"
    тест(что ,где , регистр, ожидаемый_ответ, название)

    название = "что корректное, где пустой текст, регистр корректный - ожидаем ошибку"
    что = ["1", "2", "3"]
    где = ""
    регистр = True
    ожидаемый_ответ = "ошибка"
    тест(что ,где , регистр, ожидаемый_ответ, название)

    название = "что корректное, где один из элементов не строка, регистр корректный - ожидаем ошибку"
    что = ["1", "2", "3"]
    где = ["1", "2", 3]
    регистр = True
    ожидаемый_ответ = "ошибка"
    тест(что ,где , регистр, ожидаемый_ответ, название)


    название = "что корректное, где корректное, регистр True - не ожидаем ошибку"
    что = ["1", "2", "3"]
    где = ["1", "2", "3"]
    регистр = True
    ожидаемый_ответ = True
    тест(что ,где , регистр, ожидаемый_ответ, название)

    название = "что корректное, где корректное, регистр False - не ожидаем ошибку"
    что = ["1", "2", "3"]
    где = ["1", "2", "3"]
    регистр = False
    ожидаемый_ответ = True
    тест(что ,где , регистр, ожидаемый_ответ, название)


    название = "что корректное, где корректное, регистр другой тип - ожидаем ошибку"
    что = ["1", "2", "3"]
    где = ["1", "2", "3"]
    регистр = "True"
    ожидаемый_ответ = "ошибка"
    тест(что ,где , регистр, ожидаемый_ответ, название)

    
    список_видео = ["фильм.mp4", "кино.mp4", "песня.MP3"]
    список_картинок = ["котики.jpg", "море.jpg", "каникулы.mp4"]
    список_музыки = ["мелодия.mp3", "трек.mp3", "обложка.JPG"]
    список_документов = ["доклад.docx", "список дел.txt", "книга.fb2", "таблица.xlsx"]

    варианты = [
        ["элемент-элемент, регистр важен, результат - не содержит", ".jpg", "обложка.JPG", False, False],
        ["элемент-элемент, регистр важен, результат - содержит", ".jpg", "котики.jpg", False, True],
        ["элемент-элемент, регистр не важен, результат - не содержит", ".jpg", "песня.MP3", True, False],
        ["элемент-элемент, регистр не важен, результат - содержит", ".jpg", "обложка.JPG", True, True],
        ["элемент-множество, регистр важен, результат - не содержит", ".mp3", список_видео, False, False],
        ["элемент-множество, регистр важен, результат - содержит", ".jpg", список_картинок, False, True],
        ["элемент-множество, регистр не важен, результат - не содержит", ".txt", список_видео, True, False],
        ["элемент-множество, регистр не важен, результат - содержит", ".mp3", список_видео, True, True],
        ["множество-элемент, регистр важен, результат - не содержит", [".jpg", ".mp3"], "песня.MP3", False, False],
        ["множество-элемент, регистр важен, результат - содержит", [".txt", ".mp3"], "список дел.txt", False, True],
        ["множество-элемент, регистр не важен, результат - не содержит", [".xlsx", ".fb2"], "трек.mp3", True, False],
        ["множество-элемент, регистр не важен, результат - содержит", [".jpg", ".mp3"], "песня.MP3", True, True],
        ["множество-множество, регистр важен, результат - не содержит", [".jpg", ".mp3"], список_видео, False, False],
        ["множество-множество, регистр важен, результат - содержит", [".mp4", ".jpg"], список_картинок, False, True],
        ["множество-множество, регистр не важен, результат - не содержит", [".docx", ".xlsx"], список_видео, True, False],
        ["множество-множество, регистр не важен, результат - содержит", [".mp3", ".docx"], список_видео, True, True],
    ]

    for вариант in варианты:
        название, что, где, регистр, ожидаемый_ответ = вариант
        тест(что ,где , регистр, ожидаемый_ответ, название)

    вариант = ["элемент-множество, регистр важен, результат - не содержит", ".mp3", список_видео, False, False]
    название, что, где, регистр, ожидаемый_ответ = вариант
    
    print()
    print(f"Пройдено тестов: {пройдено} / 35")

    if содержит(что, где, регистр, стоп=True) == ожидаемый_ответ:
        print("верно")
    else:
        print("неверно")

if __name__ == "__main__":
    тесты()


import colorama
colorama.init()

from colorama import Fore

def печать(
        текст,
        красный=False,
        оранжевый=False,
        жёлтый=False,
        зелёный=False,
        голубой=False,
        синий=False,
        фиолетовый=False,
        стиль=None,
        не_печатать=False
):

    def определить_стиль(стиль):

        if not стиль:
            return ""
        СТИЛИ = {
            "ж" : 1, # жирный
            "к" : 3, # курсивный
            "ч" : 4, # подчёркнутый
            "з" : 9, # зачёркнутый
        }

        # при комбинирование жирного и зачёркнутого должен получится результат
        # ;1;9
        # курсивный и подчеркнутый - ;3;4
        # жирный и курсивный - ;1;3

        накопитеьная = ""
        for буква in стиль:
            СТИЛИ.get(буква)
            код_стиля = СТИЛИ.get(буква)
            этот_стиль = f";{код_стиля}"
            накопитеьная += этот_стиль
        return "\x1B[" + накопитеьная[1:] + "m"

    # определенные символы в начале и в конце написаны на C++ и отвечают за кодировку цвета

    # первая часть строки задаёт стиль цвета и формата
    # \x1B[  специальный набор символов который включает этот режим настройки
    # 255;0;0; - цвет в формате RGB
    # 4 - подчеркнутый стиль шрифта
    # m - завершает настройку шрифта

    # последняя часть строки сбрасывает стиль в стандарт чтобы всё да
    # результат = "\x1B[38;2;255;0;0;4m" + текст + "\x1B[0m"

    готовый_стиль = определить_стиль(стиль)

    if красный:
        цвет = colorama.Fore.RED
    elif оранжевый:
        цвет = "255;100;0"
    elif жёлтый:
        цвет = colorama.Fore.YELLOW
    elif зелёный:
        цвет = colorama.Fore.GREEN
    elif голубой:
        цвет = colorama.Fore.CYAN
    elif синий:
        цвет = colorama.Fore.BLUE
    elif фиолетовый:
        цвет = colorama.Fore.MAGENTA
    else:
        цвет = "255;255;255"

    if цвет[-1] == "m":
        результат = f"{цвет}{готовый_стиль}{текст}\x1B[0m"
    else:
        результат = f"\x1B[38;2;{цвет}m{готовый_стиль}{текст}\x1B[0m"

    if не_печатать:
        return результат
    print(результат)

# Если оставить строки с запуском этих функций внутри файла с модулем то они будут выполнены в момент ИМПОРТА этого модуля.
# Чтобы этого не допустить но при этом оставить возможность тестировать этот файл модуля "прямо здесь" например чтобы провести отладку надо эти строки убрать внутрь блока если который будет проверять откуда закончить файл

# ЕСЛИ запуск произошел из этого файла (если он не импортируется)
if __name__ == "__main__":
    печать("Привет", зелёный=True)
    печать("Привет", оранжевый=True)
    печать("Привет", фиолетовый=True)
    печать("Привет", синий=True)
    
    print( )

    красный = печать("Каждый", красный=True, не_печатать=True)
    оранжевый = печать("охотник", оранжевый=True, не_печатать=True)
    жёлтый = печать("желает", жёлтый=True, не_печатать=True)
    зелёный = печать("знать", зелёный=True, не_печатать=True)
    голубой = печать("где", голубой=True, не_печатать=True)
    синий = печать("сидит", синий=True, не_печатать=True)
    фиолетовый = печать("фазан", фиолетовый=True, не_печатать=True)
    разноцветная_строка = f"{красный} {оранжевый} {жёлтый} {зелёный} {голубой} {синий} {фиолетовый}"
    print(разноцветная_строка)

    печать("красный (колорама)", красный=True, стиль="жч")
    печать("оранжевый (свой цвет)", оранжевый=True, стиль="жч")



# pip install pytz
# pip install python-dotenv
# pip install python-telegram-bot==13.7

import os
import time
from datetime import datetime
from pytz import UTC
from dotenv import load_dotenv
from telegram import Bot

RED = '\x1b[31m'
GREEN = '\x1b[32m'
YELLOW = '\x1b[33m'
RESET = '\x1b[39m'

load_dotenv()
ТОКЕН = "6441820101:AAFVP0IapyOHIzOiOR1vjKFUtLrZCQKqwVQ"
if not ТОКЕН:
    raise KeyError('Не найден файл .env')
БОТ = Bot(ТОКЕН)
этот_файл = __file__
this_folder = os.path.dirname(этот_файл)

def отправить_сообщение(сообщение, айди, кнопки=None):
    if type(айди) is int:
        try:
            БОТ.send_message(chat_id=айди, text=сообщение, reply_markup=кнопки)
        except Exception as error:
            print(f"Возникла ошибка {RED}{error}{RESET}")
    else:
        print(сообщение)

def проверить_входящие(айди=None, максимальное_ожидание_ответа_в_секундах=None):
    def отмена_если_нет_ответа(now):
        if максимальное_ожидание_ответа_в_секундах and (datetime.now(UTC) - now).seconds > максимальное_ожидание_ответа_в_секундах:
            print(f"Не было ответа в течении {максимальное_ожидание_ответа_в_секундах} сек. Перестаём следить за этим юзером")
            return True

    def создать_last_msg(update_id):
        with open(f'{this_folder}\\last_msg', 'w', encoding='utf-8') as file:
            file.write(str(update_id))

    def считать_last_msg():
        with open(f'{this_folder}\\last_msg', 'r', encoding='utf-8') as file:
            return int(file.read())

    now = datetime.now(UTC)
    if not os.path.exists(f'{this_folder}\\last_msg'):
        while True:
            try:
                обновления = БОТ.get_updates()
                if отмена_если_нет_ответа(now):
                    return None
                if not обновления:
                    time.sleep(0.5)
                    continue
                break
            except:
                continue
        last_msg = обновления[-1]
        if last_msg.message.date < now:
            создать_last_msg(last_msg.update_id)
        else:
            создать_last_msg(last_msg.update_id - 1)

    while True:
        if отмена_если_нет_ответа(now):
            return None
        try:
            обновления = БОТ.get_updates(offset=считать_last_msg() + 1)
        except:
            time.sleep(1)
            continue
        if not обновления:
            continue

        обновления.reverse()
        print(len(обновления))
        for msg in обновления:
            if айди is None or msg.effective_user.id == айди:
                создать_last_msg(msg.update_id)
                print(msg.effective_message.text)
                return msg
        time.sleep(0.5)

if __name__ == "__main__":
    проверить_входящие(478735661)